@@ -6,6 +6,7 @@ using System;
 using FSpot;
 using FSpot.Xmp;
 using System.IO;
+using Mono.Unix;
 
 public class ImportException : System.Exception {
 	public ImportException (string msg) : base (msg)
@@ -15,11 +16,13 @@ public class ImportException : System.Exception {
 
 public class FileImportBackend : ImportBackend {
 	PhotoStore store;
+ 	RollStore rolls = FSpot.Core.Database.Rolls;
 	TagStore tag_store = FSpot.Core.Database.Tags;
 	bool recurse;
 	bool copy;
 	string [] base_paths;
 	Tag [] tags;
+	Gtk.Window parent;
 
 	int count;
 	XmpTagsImporter xmptags;
@@ -119,6 +122,8 @@ public class FileImportBackend : ImportBackend {
 		directories = new Stack ();
 		xmptags = new XmpTagsImporter (store, tag_store);
 
+		roll = rolls.Create ();
+
 		return import_info.Count;
 	}
 
@@ -147,10 +152,12 @@ public class FileImportBackend : ImportBackend {
 	public static string ChooseLocation (string path, Stack created_directories)
 	{
 		string name = System.IO.Path.GetFileName (path);
-		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
-		DateTime time = img.Date;
-		
-		string dest_dir = String.Format ("{0}{1}{2}{1}{3}{1}{4}",
+		DateTime time;
+		using (FSpot.ImageFile img = FSpot.ImageFile.Create (path)) {
+			time = img.Date;
+		}
+
+		string dest_dir = String.Format ("{0}{1}{2}{1}{3:D2}{1}{4:D2}",
 						 FSpot.Global.PhotoDirectory,
 						 System.IO.Path.DirectorySeparatorChar,
 						 time.Year,
@@ -163,7 +170,7 @@ public class FileImportBackend : ImportBackend {
 			// event of a cancel.
 			if (created_directories != null) {
 				string [] parts = dest_dir.Split (new char [] {'/'});
-				string nextPath = "";
+				string nextPath = String.Empty;
 				for (int i = 0; i < parts.Length; i++) {
 					if (i == 0)
 						nextPath += parts [i];
@@ -192,6 +199,8 @@ public class FileImportBackend : ImportBackend {
 
 	public override bool Step (out Photo photo, out Pixbuf thumbnail, out int count)
 	{
+		thumbnail = null;
+
 		if (import_info == null)
 			throw new ImportException ("Prepare() was not called");
 
@@ -201,7 +210,7 @@ public class FileImportBackend : ImportBackend {
 		// FIXME Need to get the EXIF info etc.
 		ImportInfo info = (ImportInfo)import_info [this.count];
 		bool needs_commit = false;
-
+		bool abort = false;
 		try {
 			string destination = info.OriginalPath;
 			if (copy)
@@ -210,12 +219,12 @@ public class FileImportBackend : ImportBackend {
 			// Don't copy if we are already home
 			if (info.OriginalPath == destination) {
 				info.DestinationPath = destination;
-				photo = store.Create (info.DestinationPath, out thumbnail);
+				photo = store.Create (info.DestinationPath, roll.Id, out thumbnail);
 			} else {
 				System.IO.File.Copy (info.OriginalPath, destination);
 				info.DestinationPath = destination;
 
-				photo = store.Create (info.DestinationPath, info.OriginalPath, out thumbnail);
+				photo = store.Create (info.DestinationPath, info.OriginalPath, roll.Id, out thumbnail);
 
 				try {
 					File.SetAttributes (destination, File.GetAttributes (info.DestinationPath) & ~FileAttributes.ReadOnly);
@@ -242,15 +251,30 @@ public class FileImportBackend : ImportBackend {
 			
 			info.Photo = photo;
 		} catch (System.Exception e) {
-			System.Console.WriteLine ("Error importing {0}\n{1}", info.OriginalPath, e.ToString ());
+			System.Console.WriteLine ("Error importing {0}{2}{1}", info.OriginalPath, e.ToString (), Environment.NewLine);
+			if (thumbnail != null)
+				thumbnail.Dispose ();
+
 			thumbnail = null;
 			photo = null;
+
+			HigMessageDialog errordialog = new HigMessageDialog (parent,
+									     Gtk.DialogFlags.Modal | Gtk.DialogFlags.DestroyWithParent,
+									     Gtk.MessageType.Error,
+									     Gtk.ButtonsType.Cancel,
+									     Catalog.GetString ("Import error"),
+									     String.Format(Catalog.GetString ("Error importing {0}{2}{2}{1}"), info.OriginalPath, e.Message, Environment.NewLine ));
+			errordialog.AddButton ("Skip", Gtk.ResponseType.Reject, false);
+			ResponseType response = (ResponseType) errordialog.Run ();
+			errordialog.Destroy ();
+			if (response == ResponseType.Cancel)
+				abort = true;
 		}
 
 		this.count ++;
 		count = this.count;
 
-		return count != import_info.Count;
+		return (!abort && count != import_info.Count);
 	}
 
 	public override void Cancel ()
@@ -288,6 +312,8 @@ public class FileImportBackend : ImportBackend {
 		}
 		// Clean up just created tags
 		xmptags.Cancel();
+
+		rolls.Remove (roll);
 	}
 
 	public override void Finish ()
@@ -303,17 +329,19 @@ public class FileImportBackend : ImportBackend {
 		import_info = null;
 		xmptags.Finish();
 		count = 0;
+	//rolls.EndImport();    // Clean up the imported session.
 	}
 
-	public FileImportBackend (PhotoStore store, string [] base_paths, bool recurse) : this (store, base_paths, false, recurse, null) {}
+	public FileImportBackend (PhotoStore store, string [] base_paths, bool recurse, Gtk.Window parent) : this (store, base_paths, false, recurse, null, parent) {}
 
-	public FileImportBackend (PhotoStore store, string [] base_paths, bool copy, bool recurse, Tag [] tags)
+	public FileImportBackend (PhotoStore store, string [] base_paths, bool copy, bool recurse, Tag [] tags, Gtk.Window parent)
 	{
 		this.store = store;
 		this.copy = copy;
 		this.base_paths = base_paths;
 		this.recurse = recurse;
 		this.tags = tags;
+		this.parent = parent;
 	}
 
 #if TEST_FILE_IMPORT_BACKEND
@@ -330,7 +358,7 @@ public class FileImportBackend : ImportBackend {
 
 		Db db = new Db (path, true);
 
-		FileImportBackend import = new FileImportBackend (db.Photos, args [0],true);
+		FileImportBackend import = new FileImportBackend (db.Photos, args [0],true, this);
 
 		Console.WriteLine ("Preparing...");
 
@@ -349,6 +377,9 @@ public class FileImportBackend : ImportBackend {
 			ongoing = import.Step (out photo, out thumbnail, out count);
 
 			Console.WriteLine ("{0}/{1} - {2}", count, total_count, photo.Path);
+
+			if (thumbnail != null)
+				thumbnail.Dispose ();
 		} while (ongoing);
 
 		import.Finish ();
