@@ -30,7 +30,7 @@ using ICSharpCode.SharpZipLib.Zip;
 using ICSharpCode.SharpZipLib.GZip;
 
 namespace FSpot {
-	public class FolderExport : GladeDialog {
+	public class FolderExport : GladeDialog, FSpot.Extensions.IExporter {
 		IBrowsableCollection selection;
 		[Glade.Widget] Gtk.ScrolledWindow thumb_scrolledwindow;
 		[Glade.Widget] Gtk.Entry name_entry;
@@ -66,7 +66,9 @@ namespace FSpot {
 		FSpot.ThreadProgressDialog progress_dialog;
 		System.Threading.Thread command_thread;
 		
-		public FolderExport (IBrowsableCollection selection) : base ("folder_export_dialog")
+		public FolderExport () : base ("folder_export_dialog")
+		{}
+		public void Run (IBrowsableCollection selection)
 		{
 			/*
 			Gnome.Vfs.ModuleCallbackFullAuthentication auth = new Gnome.Vfs.ModuleCallbackFullAuthentication ();
@@ -195,8 +197,8 @@ namespace FSpot {
 						progress_dialog.ProgressText = System.String.Format (Catalog.GetString ("{0} of {1}"), photo_index, selection.Count);
 					}
 					catch (Exception e) {
-						progress_dialog.Message = String.Format (Catalog.GetString ("Error uploading picture \"{0}\" to Gallery:\n{1}"), 
-							selection[photo_index].Name, e.Message);
+						progress_dialog.Message = String.Format (Catalog.GetString ("Error uploading picture \"{0}\" to Gallery:{2}{1}"), 
+							selection[photo_index].Name, e.Message, Environment.NewLine);
 						progress_dialog.ProgressText = Catalog.GetString ("Error");
 
 						if (progress_dialog.PerformRetrySkip ())
@@ -205,6 +207,10 @@ namespace FSpot {
 		
 				}
 
+				//create the zip tarballs for original
+				if (gallery is OriginalGallery && (bool)Preferences.Get(Preferences.EXPORT_FOLDER_INCLUDE_TARBALLS))
+					(gallery as OriginalGallery).CreateZip ();
+
 				// we've created the structure, now if the destination was local we are done
 				// otherwise we xfer 
 				if (!dest.IsLocal) {
@@ -398,14 +404,14 @@ namespace FSpot {
 
 			public ScaleRequest (string name, int width, int height, bool skip, bool exif)
 			{
-				this.Name = name != null ? name : "";
+				this.Name = name != null ? name : String.Empty;
 				this.Width = width;
 				this.Height = height;
 				this.Skip = skip;
 				this.CopyExif = exif;
 			}
 
-			public static ScaleRequest Default = new ScaleRequest ("", 0, 0, false);
+			public static ScaleRequest Default = new ScaleRequest (String.Empty, 0, 0, false);
 
 			public bool AvoidScale (int size) { 
 				return (size < this.Width && size < this.Height && this.Skip);
@@ -443,15 +449,6 @@ namespace FSpot {
 			MakeDir (SubdirPath (req.Name));
 			path = SubdirPath (req.Name, ImageName (image_num));
 			
-//			if (scale) {
-//				//FIXME we have filters for that !
-//				PixbufUtils.Resize (photo_path, path, size, true); 	
-//			} else if (rotate && (new Filters.OrientationFilter ().Convert (photo_path, path))) {
-//				// do nothing, all is well the filter did the copying
-//			} else {
-//				File.Copy (photo_path, path, true);
-//			}
-
 			using (Filters.FilterRequest request = new Filters.FilterRequest (photo.DefaultVersionUri)) {
 				filter_set.Convert (request);
 				if (request.Current.LocalPath == path)
@@ -467,40 +464,29 @@ namespace FSpot {
 								      UriList.PathToFileUriEscaped (path).ToString ());
 				}
 
-				
-				Gdk.Pixbuf img = null;
-				Gdk.Pixbuf scaled = null;
-				
 				using (Exif.ExifData data = new Exif.ExifData (photo_path)) {
 					for (int i = 1; i < requests.Length; i++) {
 						
 						req = requests [i];
 						if (scale && req.AvoidScale (size))
 							continue;
-					
-						if (img == null) {
-							ImageFile imgf = ImageFile.Create (photo.DefaultVersionUri);
-							scaled = imgf.Load (req.Width, req.Height);
-						} else
-							scaled = PixbufUtils.ScaleToMaxSize (img, req.Width, req.Height, false);
+				
+						Filters.FilterSet req_set = new Filters.FilterSet ();
+						req_set.Add (new Filters.ResizeFilter ((uint)Math.Max (req.Width, req.Height)));
+						if ((bool)Preferences.Get (Preferences.EXPORT_FOLDER_SHARPEN)) {
+							if (req.Name == "lq")
+								req_set.Add (new Filters.SharpFilter (0.1, 2, 4));
+							if (req.Name == "thumbs")
+								req_set.Add (new Filters.SharpFilter (0.1, 2, 5));
+						}
+						using (Filters.FilterRequest tmp_req = new Filters.FilterRequest (photo.DefaultVersionUri)) {
+							req_set.Convert (tmp_req);
+							MakeDir (SubdirPath (req.Name));
+							path = SubdirPath (req.Name, ImageName (image_num));
+							System.IO.File.Copy (tmp_req.Current.LocalPath, path, true);
+						}
 						
-						MakeDir (SubdirPath (req.Name));
-						path = SubdirPath (req.Name, ImageName (image_num));
-						
-						if (req.CopyExif && data != null && data.Handle.Handle == System.IntPtr.Zero) {
-							PixbufUtils.SaveJpeg (scaled, path, 90, data);
-						} else 
-							scaled.Savev (path, "jpeg", pixbuf_keys, pixbuf_values);
-						
-						if (img != null)
-							img.Dispose ();
-						
-						img = scaled;
 					}
-					
-					if (scaled != null)
-						scaled.Dispose ();
-
 				}
 			}
 		}
@@ -605,7 +591,6 @@ namespace FSpot {
 		{
 			base.GenerateLayout ();
 			MakeDir (SubdirPath ("comments"));
-			MakeDir (SubdirPath ("zip"));
 			CreateHtaccess();
 			CreateInfo();
 			SetTime ();
@@ -628,7 +613,15 @@ namespace FSpot {
 					CreateComments (collection [i].DefaultVersionUri.LocalPath, i);
 
 				Directory.SetLastWriteTimeUtc(gallery_path, collection [0].Time);
+			} catch (System.Exception e) {
+				System.Console.WriteLine (e.ToString ());
+			} 
+		}
 
+		internal void CreateZip () 
+		{
+			MakeDir (SubdirPath ("zip"));
+			try {
 				if (System.IO.Directory.Exists (SubdirPath ("mq")))
 				    CreateZipFile("mq");
 
@@ -644,7 +637,7 @@ namespace FSpot {
 		{
 			StreamWriter comment = File.CreateText(SubdirPath  ("comments", photo_index + 1 + ".txt"));
 			comment.Write("<span>photo " + (photo_index + 1) + "</span> ");
-			comment.Write (collection [photo_index].Description + "\n");
+			comment.Write (collection [photo_index].Description + Environment.NewLine);
 			comment.Close();
 		}
 
@@ -691,7 +684,7 @@ namespace FSpot {
 		private void CreateHtaccess()
 		{
 			StreamWriter htaccess = File.CreateText(Path.Combine (gallery_path,".htaccess"));
-			htaccess.Write("<Files info.txt>\n\tdeny from all\n</Files>\n");
+			htaccess.Write("<Files info.txt>" + Environment.NewLine + "\tdeny from all" + Environment.NewLine+ "</Files>" + Environment.NewLine);
 			htaccess.Close();
 		}
 
@@ -916,7 +909,7 @@ namespace FSpot {
 			writer.Write ("<span class=\"style_toggle\">"); 
 			writer.Write ("<a href=\"javascript:toggle_stylebox()\">");
 			writer.Write ("<span id=\"showlink\">" + Catalog.GetString ("Show Styles") + "</span><span id=\"hidelink\" ");
-			writer.Write ("style=\"display:none;\">" + Catalog.GetString ("Hide Styles") + "</span></a></span>\n");
+			writer.Write ("style=\"display:none;\">" + Catalog.GetString ("Hide Styles") + "</span></a></span>" + Environment.NewLine);
 			writer.RenderEndTag (); //div toggle
 			writer.RenderEndTag (); //div styleboxcontainer
 			writer.RenderEndTag (); //container1	
@@ -955,15 +948,15 @@ namespace FSpot {
 
 			writer.Write ("<link type=\"text/css\" rel=\"stylesheet\" href=\"");
 			writer.Write (String.Format ("{0}", "style/" + stylesheet));
-			writer.Write ("\" title=\"" + dark + "\" media=\"screen\" />\n");
+			writer.Write ("\" title=\"" + dark + "\" media=\"screen\" />" + Environment.NewLine);
 
 			writer.Write ("<link type=\"text/css\" rel=\"prefetch ") ;
 			writer.Write ("alternate stylesheet\" href=\"");
 			writer.Write (String.Format ("{0}", "style/" + altstylesheet));
-			writer.Write ("\" title=\"" + light + "\" media=\"screen\" />\n");
+			writer.Write ("\" title=\"" + light + "\" media=\"screen\" />" + Environment.NewLine);
 
 			writer.Write ("<script src=\"script/" + javascript + "\"");
-			writer.Write (" type=\"text/javascript\"></script>\n");
+			writer.Write (" type=\"text/javascript\"></script>" + Environment.NewLine);
 
 			writer.RenderEndTag ();
 		}
@@ -1089,7 +1082,7 @@ namespace FSpot {
 			writer.Write ("<span class=\"style_toggle\">"); 
 			writer.Write ("<a href=\"javascript:toggle_stylebox()\">");
 			writer.Write ("<span id=\"showlink\">" + Catalog.GetString("Show Styles") + "</span><span id=\"hidelink\" ");
-			writer.Write ("style=\"display:none;\">" + Catalog.GetString("Hide Styles") + "</span></a></span>\n");
+			writer.Write ("style=\"display:none;\">" + Catalog.GetString("Hide Styles") + "</span></a></span>" + Environment.NewLine);
 			writer.RenderEndTag (); //div toggle
 			writer.RenderEndTag (); //div styleboxcontainer
 			writer.RenderEndTag (); //container1
