@@ -1,3 +1,12 @@
+/*
+ * FSpot.FullScreenView
+ *
+ * Author(s):
+ * 	Larry Ewing  <lewing@novell.com>
+ *
+ * This is free software. See COPYING for details.
+ */
+
 using System;
 using Gtk;
 using Gdk;
@@ -15,6 +24,7 @@ namespace FSpot {
 		private ControlOverlay controls;
 		//		private ImageDisplay display;
 		private TextureDisplay display;
+		private ToolButton play_pause_button;
 
 		ActionGroup actions;
 		const string ExitFullScreen = "ExitFullScreen";
@@ -24,6 +34,13 @@ namespace FSpot {
 		
 		public FullScreenView (IBrowsableCollection collection) : base ("Full Screen Mode")
 		{
+			string style = "style \"test\" {\n" +
+				"GtkToolbar::shadow_type = GTK_SHADOW_NONE\n" +
+				"}\n" +
+				"class \"GtkToolbar\" style \"test\"";
+
+			Gtk.Rc.ParseString (style);
+
 			Name = "FullscreenContainer";
 			try {
 				//scroll = new Gtk.ScrolledWindow (null, null);
@@ -34,20 +51,7 @@ namespace FSpot {
 							 Catalog.GetString ("Hide"), 
 							 null, 
 							 Catalog.GetString ("Hide Toolbar"), 
-							 HideToolbarAction),
-					new ActionEntry (ExitFullScreen, 
-							 "f-spot-view-restore", 
-							 Catalog.GetString ("Exit fullscreen"), 
-							 null, 
-							 null, 
-							 ExitAction),
-					new ActionEntry (SlideShow,
-							 "f-spot-slideshow",
-							 Catalog.GetString ("Slideshow"),
-							 null,
-							 Catalog.GetString ("Start slideshow"),
-							 SlideShowAction),
-						});
+							 HideToolbarAction)});
 
 				actions.Add (new ToggleActionEntry [] {
 					new ToggleActionEntry (Info,
@@ -56,9 +60,28 @@ namespace FSpot {
 							       null,
 							       Catalog.GetString ("Image Information"),
 							       InfoAction,
-							       false)
-						});
-				
+							       false)});
+
+				Action exit_full_screen = new Action (ExitFullScreen, 
+					Catalog.GetString ("Exit fullscreen"),
+					null,
+					null);
+#if GTK_2_10
+				exit_full_screen.IconName = "view-restore";
+#endif
+				exit_full_screen.Activated += ExitAction;
+				actions.Add (exit_full_screen);
+
+				Action slide_show = new Action (SlideShow,
+					Catalog.GetString ("Slideshow"),
+					Catalog.GetString ("Start slideshow"),
+					null);
+#if GTK_2_10
+				slide_show.IconName = "media-playback-start";
+#endif
+				slide_show.Activated += SlideShowAction;
+				actions.Add (slide_show);
+
 				new Fader (this, 1.0, 3);
 				notebook = new Notebook ();
 				notebook.ShowBorder = false;
@@ -66,6 +89,7 @@ namespace FSpot {
 				notebook.Show ();
 
 				scroll = new ScrolledView ();
+				scroll.ScrolledWindow.SetPolicy (PolicyType.Never, PolicyType.Never);
 				view = new PhotoImageView (collection);
 				// FIXME this should be handled by the new style setting code
 				view.ModifyBg (Gtk.StateType.Normal, this.Style.Black);
@@ -74,45 +98,93 @@ namespace FSpot {
 				view.Show ();
 				view.MotionNotifyEvent += HandleViewMotion;
 				
-				Action rotate_left = new RotateLeftAction (view.Item);
-				actions.Add (rotate_left);
-				
-				Action rotate_right = new RotateRightAction (view.Item);
-				actions.Add (rotate_right);
-
 				scroll.ScrolledWindow.Add (view);
-				HBox hhbox = new HBox ();
-				hhbox.PackEnd (GetButton (HideToolbar), false, true, 0);
-				hhbox.PackEnd (GetButton (Info), false, true, 0);
-				hhbox.PackStart (GetButton (ExitFullScreen, true), false, false, 0);
-				hhbox.PackStart (Add (new PreviousPictureAction (view.Item)), false, false, 0);
-				hhbox.PackStart (GetButton (SlideShow), false, true, 0);
-				hhbox.PackStart (Add (new NextPictureAction (view.Item)), false, false, 0);
-				//hhbox.PackStart (Add (new AutoColor (view.Item)), false, false, 0);
+
+				Toolbar tbar = new Toolbar ();
+				tbar.ToolbarStyle = Gtk.ToolbarStyle.BothHoriz;
+
+				tbar.ShowArrow = false;
+				tbar.BorderWidth = 15;
+
+				ToolItem t_item = (actions [ExitFullScreen]).CreateToolItem () as ToolItem;
+				t_item.IsImportant = true;
+				tbar.Insert (t_item, -1);
+
+				Action action = new PreviousPictureAction (view.Item);
+				actions.Add (action);
+#if GTK_2_10
+				tbar.Insert (action.CreateToolItem () as ToolItem, -1);
+#else
+				t_item = action.CreateToolItem () as ToolItem;
+				(t_item as ToolButton).IconName = "gtk-go-back-ltr"; 
+				tbar.Insert (t_item, -1);
+#endif
+
+				play_pause_button = (actions [SlideShow]).CreateToolItem () as ToolButton;
+#if GTK_2_10
+				tbar.Insert (play_pause_button, -1);
+#else
+				play_pause_button.IconName = "media-playback-start";
+				tbar.Insert (play_pause_button, -1);
+#endif
+
+				action = new NextPictureAction (view.Item);
+				actions.Add (action);
+#if GTK_2_10
+				tbar.Insert (action.CreateToolItem () as ToolItem, -1);
+#else
+				t_item = action.CreateToolItem () as ToolItem;
+				(t_item as ToolButton).IconName = "gtk-go-forward-ltr"; 
+				tbar.Insert (t_item, -1);
+#endif
+
+				t_item = new ToolItem ();
+				t_item.Child = new Label (Catalog.GetString ("Slide transition: "));
+				tbar.Insert (t_item, -1);
 
 				display = new TextureDisplay (view.Item);
 				display.AddEvents ((int) (Gdk.EventMask.PointerMotionMask));
 				display.ModifyBg (Gtk.StateType.Normal, this.Style.Black);
 				display.MotionNotifyEvent += HandleViewMotion;
-				Label effect = new Label (Catalog.GetString ("Slide transition: "));
-				hhbox.PackStart (effect, false, false, 5);
-				hhbox.PackStart (display.GetCombo (), false, false, 0);
 				display.Show ();
 
-				hhbox.PackStart (Add (new RotateLeftAction (view.Item)), false, false, 0);
-				hhbox.PackStart (Add (new RotateRightAction (view.Item)), false, false, 0);
-				hhbox.BorderWidth = 15;
+				t_item = new ToolItem ();
+				t_item.Child = display.GetCombo ();
+				tbar.Insert (t_item, -1);
+
+				action = new RotateLeftAction (view.Item);
+				actions.Add (action);
+#if GTK_2_10
+				tbar.Insert (action.CreateToolItem () as ToolItem, -1);
+#else
+				t_item = action.CreateToolItem () as ToolItem;
+				(t_item as ToolButton).IconName = "object-rotate-left"; 
+				tbar.Insert (t_item, -1);
+#endif
+
+				action = new RotateRightAction (view.Item);
+				actions.Add (action);
+#if GTK_2_10
+				tbar.Insert (action.CreateToolItem () as ToolItem, -1);
+#else
+				t_item = action.CreateToolItem () as ToolItem;
+				(t_item as ToolButton).IconName = "object-rotate-right"; 
+				tbar.Insert (t_item, -1);
+#endif
 
 				tag_view = new TagView ();
-				hhbox.PackStart (tag_view, false, false, 0);
+				t_item = new ToolItem ();
+				t_item.Child = tag_view;
+				tbar.Insert (t_item, -1);
 
-				//display = new ImageDisplay (view.Item);
+				tbar.Insert ((actions [Info]).CreateToolItem () as ToolItem, -1);
+
+				tbar.Insert ((actions [HideToolbar]).CreateToolItem () as ToolItem, -1);
 
 				notebook.AppendPage (scroll, null);
 				notebook.AppendPage (display, null);
 
-				hhbox.ShowAll ();
-				//scroll.ShowControls ();
+				tbar.ShowAll ();
 				
 				scroll.Show ();
 				this.Decorated = false;
@@ -123,7 +195,7 @@ namespace FSpot {
 				view.GrabFocus ();
 				
 				controls = new ControlOverlay (this);
-				controls.Add (hhbox);
+				controls.Add (tbar);
 				controls.Dismiss ();
 
 				notebook.CurrentPage = 0;
@@ -133,12 +205,6 @@ namespace FSpot {
 
 		}
 
-		private Widget Add (Action action)
-		{
-			actions.Add (action);
-			return GetButton (action.Name);
-		}
-
 		private void HandleItemChanged (object sender, BrowsablePointerChangedArgs args)
 		{
 			tag_view.Current = view.Item.Current;
@@ -159,40 +225,7 @@ namespace FSpot {
 			return ret;
 		}
 #endif
-		private Button GetButton (string name)
-		{
-			return GetButton (name, false);
-		}
-
-		private Button GetButton (string name, bool label)
-		{
-			Action action = actions [name];
-			Widget w = action.CreateIcon (IconSize.LargeToolbar);
-			if (label) {
-				HBox box = new HBox ();
-				box.PackStart (w, false, false, 0);
-				Label l = new Label ();
-				//l.Markup = "<small>" + action.Label + "</small>";
-				l.Text = action.Label;
-				box.PackStart (l);
-				w = box;
-			}
-			Button button;
-			if (action is ToggleAction) {
-				ToggleButton toggle = new ToggleButton ();
-				toggle.Active = ((ToggleAction)action).Active;
-				button = toggle;
-			} else {
-				button = new Button ();
-			}
-			button.Relief = ReliefStyle.None;
-			button.Add (w);
-			w.ShowAll ();
-
-			action.ConnectProxy (button);
-			return button;
-		}
-			
+		
 		private void ExitAction (object sender, System.EventArgs args)
 		{
 			this.Destroy ();
@@ -250,10 +283,13 @@ namespace FSpot {
 
 		public bool PlayPause ()
 		{
-			if (notebook.CurrentPage == 0)
+			if (notebook.CurrentPage == 0) {
 				notebook.CurrentPage = 1;
-			else
+				play_pause_button.IconName = "media-playback-pause";
+			} else {
 				notebook.CurrentPage = 0;
+				play_pause_button.IconName = "media-playback-start";
+			}
 			return true;
 		}
 
@@ -264,7 +300,22 @@ namespace FSpot {
 
 		protected override bool OnKeyPressEvent (Gdk.EventKey key)
 		{
-			controls.Visibility = ControlOverlay.VisibilityType.Partial;
+			switch (key.Key) {
+				case Gdk.Key.Up:
+				case Gdk.Key.Left:
+				case Gdk.Key.KP_Up:
+				case Gdk.Key.KP_Left:
+				case Gdk.Key.Page_Up:
+				case Gdk.Key.Down:
+				case Gdk.Key.Right:
+				case Gdk.Key.KP_Down:
+				case Gdk.Key.KP_Right:
+				case Gdk.Key.Page_Down:
+					break;
+				default:
+					controls.Visibility = ControlOverlay.VisibilityType.Partial;
+					break;
+			}
 
 			if (key == null) {
 				System.Console.WriteLine ("Key == null", key);
