@@ -39,8 +39,10 @@ namespace FSpot {
 		protected FSpot.GroupAdaptor adaptor;
 		public FSpot.GroupAdaptor Adaptor {
 			set {
-				if (adaptor != null)
+				if (adaptor != null) {
 					adaptor.Changed -= HandleAdaptorChanged;
+					adaptor.Dispose ();
+				}
 
 				adaptor = value;
 				HandleAdaptorChanged (adaptor);
@@ -52,14 +54,14 @@ namespace FSpot {
 				}
 			
 				if (adaptor is TimeAdaptor) {
-					MainWindow.SetTip (left, Catalog.GetString ("More dates"));
-					MainWindow.SetTip (right, Catalog.GetString ("More dates"));
+					MainWindow.ToolTips.SetTip (left, Catalog.GetString ("More dates"), null);
+					MainWindow.ToolTips.SetTip (right, Catalog.GetString ("More dates"), null);
 				} else if (adaptor is DirectoryAdaptor) {
-					MainWindow.SetTip (left, Catalog.GetString ("More directories"));
-					MainWindow.SetTip (right, Catalog.GetString ("More directories"));
+					MainWindow.ToolTips.SetTip (left, Catalog.GetString ("More directories"), null);
+					MainWindow.ToolTips.SetTip (right, Catalog.GetString ("More directories"), null);
 				} else {
-					MainWindow.SetTip (left, Catalog.GetString ("More"));
-					MainWindow.SetTip (right, Catalog.GetString ("More"));
+					MainWindow.ToolTips.SetTip (left, Catalog.GetString ("More"), null);
+					MainWindow.ToolTips.SetTip (right, Catalog.GetString ("More"), null);
 				}
 				
 				adaptor.Changed += HandleAdaptorChanged;
@@ -488,7 +490,7 @@ namespace FSpot {
 					layout.GetPixelSize (out width, out height);
 					
 					Style.PaintLayout (Style, GdkWindow, State, true, area, this,
-						   "GroupSelector:Tick", tick.X, tick.Y + tick.Height, layout); 
+						   "GroupSelector:Tick", tick.X + 3, tick.Y + tick.Height, layout); 
 				}
 			}
 
@@ -693,12 +695,12 @@ namespace FSpot {
 			public Glass (GroupSelector selector) : base (selector) 
 			{
 				popup_window = new TipWindow ();
-				popup_label = new Gtk.Label ("");
+				popup_label = new Gtk.Label (String.Empty);
 				popup_label.Show ();
 				popup_window.Add (popup_label);
 			}
 
-			private int handle_height = 15;
+			public int handle_height = 15;
 			private int border {
 				get {
 					return selector.box_spacing * 2;
@@ -964,7 +966,7 @@ namespace FSpot {
 		protected override void OnSizeRequested (ref Requisition requisition)
 		{
 			requisition.Width = 500;
-			requisition.Height = 70;
+			requisition.Height = (int) (LegendHeight () + glass.handle_height + 3 * border);
 			base.OnSizeRequested (ref requisition);
 		}
 
@@ -1034,7 +1036,7 @@ namespace FSpot {
 	
 			Gdk.Rectangle bar = new Rectangle (alloc.X + border, alloc.Y + border,
 							   alloc.Width - 2 *  border,
-							   alloc.Height - 2 * border - legend_height);
+							   alloc.Height - 2 * border - glass.handle_height);
 							   
 		
 			if (left.Allocation.Y != bar.Y || left.Allocation.X != bar.X) {
@@ -1052,10 +1054,10 @@ namespace FSpot {
 						    bar.Width - left.Allocation.Width - right.Allocation.Width, 
 						    bar.Height);
 			
-			legend = new Rectangle (background.X, background.Y + background.Height,
+			legend = new Rectangle (background.X, background.Y,
 						background.Width, legend_height);
 
-			action = background.Union (legend);
+			action = background.Union (glass.Bounds ());
 
 			if (event_window != null)
 				event_window.MoveResize (action.X, action.Y, action.Width, action.Height);
