@@ -98,6 +98,7 @@ namespace FSpot {
 			value = value.Replace ("&", "&amp;");
 			value = value.Replace (">", "&gt;");
 			value = value.Replace ("<", "&lt;");
+			value = value.Replace ("\r\n", "<br>");
 			value = value.Replace ("\n", "<br>");
 			return value;
 		}
@@ -112,6 +113,9 @@ namespace FSpot {
 			case ExportStore.FolderExportType:
 				Gnome.Vfs.Uri uri = new Gnome.Vfs.Uri (export.ExportToken);
 				return (uri.HasParent) ? uri.Parent.ToString () : export.ExportToken;
+			case ExportStore.Gallery2ExportType:
+				string[] split_item = export.ExportToken.Split (':');
+				return String.Format ("{0}:{1}?g2_itemId={2}",split_item[0], split_item[1], split_item[2]);
 			case ExportStore.OldFolderExportType:	//This is obsolete and meant to be removed once db reach rev4
 			case ExportStore.PicasaExportType:
 			case ExportStore.SmugMugExportType:
@@ -135,6 +139,8 @@ namespace FSpot {
 				return Catalog.GetString ("Picasaweb");
 			case ExportStore.SmugMugExportType:
 				return Catalog.GetString ("SmugMug");
+			case ExportStore.Gallery2ExportType:
+				return Catalog.GetString ("Gallery2");
 			default:
 				return null;
 			}
@@ -185,7 +191,7 @@ namespace FSpot {
 							stream.Write ("&lt;Unknown Tag ID=" + entry.Tag.ToString () + "&gt;");
 						stream.Write ("</font></small></td><td>");
 						string s = entry.Value;
-						if (s != null && s != "")
+						if (s != null && s != String.Empty)
 							stream.Write (s);
 						stream.Write ("</td><tr>");
 					}
@@ -195,10 +201,11 @@ namespace FSpot {
 			if (photo != null) {
 				MetadataStore store = new MetadataStore ();
 				try {
-					ImageFile img = ImageFile.Create (photo.DefaultVersionUri);
-					if (img is SemWeb.StatementSource) {
-						StatementSource source = (StatementSource)img;
-						source.Select (store);
+					using (ImageFile img = ImageFile.Create (photo.DefaultVersionUri)) {
+						if (img is SemWeb.StatementSource) {
+							StatementSource source = (StatementSource)img;
+							source.Select (store);
+						}
 					}
 				} catch (System.IO.FileNotFoundException) {
 					missing = true;
@@ -249,14 +256,14 @@ namespace FSpot {
 							WriteCollection (substore, stream);
 						}
 						
-						if (value != null && value != "")
+						if (value != null && value != String.Empty)
 							stream.Write (value);
 						
 						stream.Write ("</td></tr>");
 					}
 				}
-				
-				if (Core.Database != null) {
+	
+				if (Core.Database != null && photo is Photo) {
 					stream.Write ("<tr><th align=left bgcolor=\"" + ig + "\" colspan=2>" + Catalog.GetString ("Exported Locations") + "</th></tr>");
 	
 					Photo p = photo as Photo;
@@ -317,7 +324,7 @@ namespace FSpot {
 				if (sub.Object is Literal) {
 					string title;
 					string value = ((Literal)(sub.Object)).Value;
-					string vc = "";
+					string vc = String.Empty;
 					
 					Description.GetDescription (substore, sub, out title, out value);
 
@@ -383,7 +390,7 @@ namespace FSpot {
 						}
 					}
 					*/
-					if (s != null && s != "")
+					if (s != null && s != String.Empty)
 						stream.Write (s);
 					stream.Write ("</td><tr>");
 				}
