@@ -5,6 +5,7 @@ using Mono.Data.SqliteClient;
 using System.Collections;
 using System.IO;
 using System;
+using Banshee.Database;
 
 public class MetaItem : DbItem {
 	private string name;
@@ -60,18 +61,13 @@ public class MetaStore : DbStore {
 
 	private void CreateTable ()
 	{
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
+		Database.ExecuteNonQuery(     
+			"CREATE TABLE meta (					" +
+			"	id		INTEGER PRIMARY KEY NOT NULL,	" +
+			"	name		TEXT UNIQUE NOT NULL,		" +
+			"	data		TEXT				" +
+			")");
 
-		command.CommandText =
-			@"CREATE TABLE meta (
-				id		INTEGER PRIMARY KEY NOT NULL,
-				name		TEXT UNIQUE NOT NULL,
-				data		TEXT
-			)";
-
-		command.ExecuteNonQuery ();
-		command.Dispose ();
 	}
 
 	private void CreateDefaultItems (bool is_new)
@@ -80,29 +76,15 @@ public class MetaStore : DbStore {
 		Create (db_version, (is_new) ? FSpot.Database.Updater.LatestVersion.ToString () : "0");
 		
 		// Get the hidden tag id, if it exists
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = "SELECT id FROM tags WHERE name = 'Hidden'";
-		
 		try {
-			SqliteDataReader reader = command.ExecuteReader ();
-		
-			if (reader.Read ())
-				Create (hidden, reader [0].ToString ());
-
-			reader.Close ();
-		} catch (Exception e) {}
-	
-		command.Dispose ();
+			string id = (string)Database.QuerySingle("SELECT id FROM tags WHERE name = 'Hidden'");
+			Create (hidden, id);
+		} catch (Exception) {}
 	}
 	
 	private void LoadAllItems ()
 	{
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-
-		command.CommandText = "SELECT id, name, data FROM meta";
-		SqliteDataReader reader = command.ExecuteReader ();
+		SqliteDataReader reader = Database.Query("SELECT id, name, data FROM meta");
 
 		while (reader.Read ()) {
 			uint id = Convert.ToUInt32 (reader [0]);
@@ -119,7 +101,6 @@ public class MetaStore : DbStore {
 		}
 
 		reader.Close ();
-		command.Dispose ();
 
 		if (FSpotVersion.Value != FSpot.Defines.VERSION) {
 			FSpotVersion.Value = FSpot.Defines.VERSION;
@@ -129,16 +110,16 @@ public class MetaStore : DbStore {
 
 	private MetaItem Create (string name, string data)
 	{
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
 
-		command.CommandText = String.Format ("INSERT INTO meta (name, data) VALUES ('{0}', {1})",
-				name, (data == null) ? "NULL" : "'" + data + "'");
+		uint id = (uint)Database.Execute(new DbCommand("INSERT INTO meta (name, data) VALUES (:name, :data)",
+				"name", name, "data", (data == null) ? "NULL" : data ));
 		
-		MetaItem item = new MetaItem ((uint) Connection.LastInsertRowId, name, data);
+		//FIXME This smells bad. This line used to be *before* the
+		//Command.executeNonQuery. It smells of a bug, but there might
+		//have been a reason for this
+
+		MetaItem item = new MetaItem (id, name, data);
 
-		command.ExecuteScalar ();
-		command.Dispose ();
 		
 		AddToCache (item);
 		EmitAdded (item);
@@ -150,13 +131,7 @@ public class MetaStore : DbStore {
 	{
 		MetaItem item = dbitem as MetaItem;
 
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-
-		command.CommandText = String.Format ("UPDATE meta SET data = '{1}' WHERE name = '{0}'", item.Name, item.Value);
-
-		command.ExecuteNonQuery ();
-		command.Dispose ();
+		Database.ExecuteNonQuery(new DbCommand("UPDATE meta SET data = :data WHERE name = :name", "name", item.Name, "data", item.Value));
 		
 		EmitChanged (item);
 	}
@@ -170,35 +145,17 @@ public class MetaStore : DbStore {
 	{
 		RemoveFromCache (item);
 		
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
+		Database.ExecuteNonQuery (new DbCommand ("DELETE FROM meta WHERE id = :id", "id", item.Id));
 
-		command.CommandText = String.Format ("DELETE FROM meta WHERE id = {0}", item.Id);
-		command.ExecuteNonQuery ();
-
-		command.Dispose ();
 		EmitRemoved (item);
 	}
 
 	// Constructor
 
-	public MetaStore (SqliteConnection connection, bool is_new)
-		: base (connection, true)
+	public MetaStore (QueuedSqliteDatabase database, bool is_new)
+		: base (database, true)
 	{
-		// Ensure the table exists
-		bool exists = true;
-		try {
-			SqliteCommand command = new SqliteCommand ();
-			command.Connection = connection;
-			command.CommandText = "UPDATE meta SET id = 1 WHERE 1 = 2";
-			command.ExecuteScalar ();
-			command.Dispose ();
-		} catch (Exception e) {
-			// Table doesn't exist, so create it
-			exists = false;
-		}
-			
-		if (is_new || !exists) {
+		if (is_new || !Database.TableExists ("meta")) {
 			CreateTable ();
 			CreateDefaultItems (is_new);
 		} else
