@@ -182,7 +182,12 @@ namespace FSpot {
 
 		public void RemoveAccount (GoogleAccount account)
 		{
-			string keyring = Ring.GetDefaultKeyring();
+			string keyring;
+			try {
+				keyring = Ring.GetDefaultKeyring();
+			} catch {
+				return;
+			}
 			Hashtable request_attributes = new Hashtable();
 			request_attributes["name"] = keyring_item_name;
 			request_attributes["username"] = account.Username;
@@ -199,7 +204,12 @@ namespace FSpot {
 
 		public void WriteAccounts ()
 		{
-			string keyring = Ring.GetDefaultKeyring();
+			string keyring;
+			try {
+				keyring = Ring.GetDefaultKeyring();
+			} catch {
+				return;
+			}
 			foreach (GoogleAccount account in accounts) {
 				Hashtable update_request_attributes = new Hashtable();
 				update_request_attributes["name"] = keyring_item_name;
@@ -265,9 +275,10 @@ namespace FSpot {
 
 			if (show_captcha) {
 				try {
-					ImageFile img = ImageFile.Create(new Uri(captcha_exception.CaptchaUrl));
-					captcha_image.Pixbuf = img.Load();
-					token = captcha_exception.Token;
+					using  (ImageFile img = ImageFile.Create(new Uri(captcha_exception.CaptchaUrl))) {
+						captcha_image.Pixbuf = img.Load();
+						token = captcha_exception.Token;
+					}
 				} catch (Exception) {}
 			}
 
@@ -293,7 +304,7 @@ namespace FSpot {
 			password = password_entry.Text;
 			username = username_entry.Text;
 
-			add_button.Sensitive = !(password == "" || username == "");
+			add_button.Sensitive = !(password == String.Empty || username == String.Empty);
 		}
 		
 		[GLib.ConnectBefore]
@@ -378,7 +389,7 @@ namespace FSpot {
 			title = title_entry.Text;
 			public_album = public_check.Active;
 
-			if (title == "")
+			if (title == String.Empty)
 				add_button.Sensitive = false;
 			else
 				add_button.Sensitive = true;
@@ -389,11 +400,24 @@ namespace FSpot {
 		{
 			if (args.ResponseId == Gtk.ResponseType.Ok) {
 				public_album = public_check.Active;
-
-				if (public_album)
-					picasa.CreateAlbum (title, description, AlbumAccess.Public);
-				else	
-					picasa.CreateAlbum (title, description, AlbumAccess.Private);
+				
+				try {
+					if (public_album)
+						picasa.CreateAlbum (title, description, AlbumAccess.Public);
+					else	
+						picasa.CreateAlbum (title, description, AlbumAccess.Private);
+				} catch (System.Exception e) {
+					HigMessageDialog md =
+					new HigMessageDialog (Dialog, 
+							      Gtk.DialogFlags.Modal |
+							      Gtk.DialogFlags.DestroyWithParent,
+								      Gtk.MessageType.Error, Gtk.ButtonsType.Ok,
+							      Catalog.GetString ("Error while creating Album"),
+							      String.Format (Catalog.GetString ("The following error was encountered while attempting to create an album: {0}"), e.Message));
+					md.Run ();
+					md.Destroy ();
+					return;
+				}
 				export.HandleAlbumAdded (title);
 			}
 			Dialog.Destroy ();
@@ -401,8 +425,12 @@ namespace FSpot {
 	}
 
 	
-	public class GoogleExport : GladeDialog {
-		public GoogleExport (IBrowsableCollection selection) : base ("google_export_dialog")
+	public class GoogleExport : GladeDialog, FSpot.Extensions.IExporter{
+		public GoogleExport () : base ("google_export_dialog")
+		{
+		}
+
+		public void Run (IBrowsableCollection selection)
 		{
 			this.items = selection.Items;
 			album_button.Sensitive = false;
@@ -542,23 +570,23 @@ namespace FSpot {
 
 		private void Upload ()
 		{
-			try {
-				album.UploadProgress += HandleUploadProgress;
-				sent_bytes = 0;
-				approx_size = 0;
+			album.UploadProgress += HandleUploadProgress;
+			sent_bytes = 0;
+			approx_size = 0;
 
-				System.Console.WriteLine ("Starting Upload to Picasa");
+			System.Console.WriteLine ("Starting Upload to Picasa");
 
-				FilterSet filters = new FilterSet ();
-				filters.Add (new JpegFilter ());
+			FilterSet filters = new FilterSet ();
+			filters.Add (new JpegFilter ());
 
-				if (scale)
-					filters.Add (new ResizeFilter ((uint)size));
+			if (scale)
+				filters.Add (new ResizeFilter ((uint)size));
 
-				if (rotate)
-					filters.Add (new OrientationFilter ());
+			if (rotate)
+				filters.Add (new OrientationFilter ());
 
-				while (photo_index < items.Length) {
+			while (photo_index < items.Length) {
+				try {
 					IBrowsableItem item = items[photo_index];
 
 					FileInfo file_info;
@@ -579,45 +607,40 @@ namespace FSpot {
 					else
 						approx_size = sent_bytes * items.Length / (photo_index - 1);
 
-					string id = album.UploadPicture (request.Current.LocalPath, item.Description);
-					if (Core.Database != null) {
-						System.UriBuilder uri = new System.UriBuilder (album.Link);
-						uri.Path = uri.Path + "/photo";
-						//Setting the Fragment doesn't display it :( workaround follows
-						string photouri;
-						if (uri.Query == null || uri.Query == String.Empty) {
-							uri.Fragment = id;
-							photouri = uri.ToString () + uri.Fragment;
-						} else {
-							uri.Query = uri.Query.Substring (1) + "#" + id;
-							photouri = uri.ToString ();
-						}
+					PicasaPicture picture = album.UploadPicture (request.Current.LocalPath, Path.ChangeExtension (item.Name, "jpg"), item.Description);
+					if (Core.Database != null && item is Photo)
 						Core.Database.Exports.Create ((item as Photo).Id,
 									      (item as Photo).DefaultVersionId,
 									      ExportStore.PicasaExportType,
-									      photouri);
-					}
+									      picture.Link);
 
 					sent_bytes += file_info.Length;
 
 					request.Dispose ();
-				}
-				
-				progress_dialog.Message = Catalog.GetString ("Done Sending Photos");
-				progress_dialog.Fraction = 1.0;
-				progress_dialog.ProgressText = Catalog.GetString ("Upload Complete");
-				progress_dialog.ButtonLabel = Gtk.Stock.Ok;
+					//tagging
+					if (item.Tags != null)
+						foreach (Tag tag in item.Tags)
+							picture.AddTag (tag.Name);
 
-				if (browser) {
-					GnomeUtil.UrlShow (null, album.Link);
+				} catch (System.Exception e) {
+					progress_dialog.Message = String.Format (Catalog.GetString ("Error Uploading To Gallery: {0}"),
+										 e.Message);
+					progress_dialog.ProgressText = Catalog.GetString ("Error");
+					System.Console.WriteLine (e);
+
+					if (progress_dialog.PerformRetrySkip ())
+					 	photo_index--;
 				}
-			} catch (System.Exception e) {
-				progress_dialog.Message = String.Format (Catalog.GetString ("Error Uploading To Gallery: {0}"),
-									 e.Message);
-				progress_dialog.ProgressText = Catalog.GetString ("Error");
-				System.Console.WriteLine (e);
 			}
-			
+				
+			progress_dialog.Message = Catalog.GetString ("Done Sending Photos");
+			progress_dialog.Fraction = 1.0;
+			progress_dialog.ProgressText = Catalog.GetString ("Upload Complete");
+			progress_dialog.ButtonLabel = Gtk.Stock.Ok;
+
+			if (browser) {
+				GnomeUtil.UrlShow (null, album.Link);
+			}
 		}
 
 		private void HandleScaleCheckToggled (object o, EventArgs e)
@@ -718,7 +741,7 @@ namespace FSpot {
 
 				PopulateAlbumOptionMenu (account.Picasa);
 
-				status_label.Text = "";
+				status_label.Text = String.Empty;
 				album_button.Sensitive = false;
 				
 				new GoogleAccountDialog (this.Dialog, account, true, null);
