@@ -32,6 +32,8 @@ using System.Collections;
 using System.IO;
 using System;
 
+using Mono.Unix;
+
 public class TagSelectionWidget : TreeView {
 	TagSelectionWidget widget;
 	private TagStore tag_store;
@@ -45,100 +47,36 @@ public class TagSelectionWidget : TreeView {
 
 	// Selection management.
 
-	// Hash of the IDs of the selected tags.
-	private Hashtable selection;
-
 	public Tag TagAtPosition (int x, int y) 
 	{
 		TreePath path;
-		TreeIter iter;
 
 		// Work out which tag we're dropping onto
 		if (!this.GetPathAtPos (x, y, out path))
 			return null;
 
+		return TagByPath (path);
+	}
+
+	public Tag TagByPath (TreePath path) 
+	{
+		TreeIter iter;
+
 		if (!Model.GetIter (out iter, path))
 			return null;
 
-		GLib.Value value = new GLib.Value ();
-
-		Model.GetValue (iter, IdColumn, ref value);
-		uint tag_id = (uint) value;
-		Tag tag = tag_store.Get (tag_id) as Tag;
-
-		return tag;
+		return TagByIter (iter);
 	}
-
-	public void Select (Tag tag)
+	
+	public Tag TagByIter (TreeIter iter)
 	{
-		if (! selection.Contains (tag.Id))
-			selection.Add (tag.Id, tag);
-	}
-
-	public void Unselect (Tag tag)
-	{
-		selection.Remove (tag.Id);
-	}
-
-	public void UnselectTagsForCategory (Category category)
-	{
-		foreach (Tag t in category.Children) {
-			Unselect (t);
-			if (t is Category)
-				UnselectTagsForCategory (t as Category);
-		}
-	}
-
-	public bool IsSelected (Tag tag)
-	{
-		if (tag == tag_store.RootCategory)
-			return false;
-		else if (selection.ContainsKey (tag.Id))
-			return true;
-		else if (tag.Category != tag_store.RootCategory && IsSelected (tag.Category))
-			return true;
-		else
-			return false;
-	}
-
-	private void GetSelectionForCategory (ArrayList selection, Category category)
-	{
-		foreach (Tag t in category.Children) {
-			if (IsSelected (t))
-				selection.Add (t);
-			if (t is Category)
-				GetSelectionForCategory (selection, t as Category);
-		}
-	}
-
-	public delegate void SelectionChangedHandler (object me);
-	public event SelectionChangedHandler SelectionChanged;
-
-	public Tag [] TagSelection {
-		get {
-			ArrayList selection = new ArrayList ();
-			GetSelectionForCategory (selection, tag_store.RootCategory);
-
-			// For some reason an empty Tag [] causes crasher problems
-			if (selection.Count == 0)
-				return null;
-			else
-				return (Tag []) selection.ToArray (typeof (Tag));
-		}
-
-		set {
-			UnselectTagsForCategory (tag_store.RootCategory);
-
-			foreach (Tag t in value)
-				Select (t);
-
-			QueueDraw ();
-
-			if (SelectionChanged != null)
-				SelectionChanged (this);
-		}
-	}
-
+		GLib.Value val = new GLib.Value ();
+ 
+		Model.GetValue (iter, IdColumn, ref val);
+		uint tag_id = (uint) val;
+ 
+		return tag_store.Get (tag_id) as Tag;
+ 	}
 
 	// Loading up the store.
 
@@ -213,41 +151,6 @@ public class TagSelectionWidget : TreeView {
 		}
 	}
 
-
-	// Event handlers.
-
-	private void OnCellToggled (object renderer, ToggledArgs args)
-	{
-		TreePath path = new TreePath (args.Path);
-
-		TreeIter iter;
-		Model.GetIter (out iter, path);
-
-		GLib.Value value = new GLib.Value ();
-		Model.GetValue (iter, IdColumn, ref value);
-		uint tag_id = (uint) value;
-		Tag tag = tag_store.Get (tag_id) as Tag;
-
-		// Tags under an unselected category are always conceptually unselected.
-		// They appear as selected just in virtue of being children of a selected category.
-		if (! IsSelected (tag.Category)) {
-			if (IsSelected (tag))
-				Unselect (tag);
-			else
-				Select (tag);
-
-			(Model as TreeStore).EmitRowChanged (path, iter);
-
-			// Make sure that if you unselect the category the tags are unselected as well.
-			if (tag is Category)
-				UnselectTagsForCategory (tag as Category);
-		}
-
-		if (SelectionChanged != null)
-			SelectionChanged (this);
-	}
-
-
 	// Data functions.
 	private static string ToHashColor (Gdk.Color color)
 	{
@@ -269,20 +172,6 @@ public class TagSelectionWidget : TreeView {
 		*/
 	}
 
-	private void CheckBoxDataFunc (TreeViewColumn column,
-				       CellRenderer renderer,
-				       TreeModel model,
-				       TreeIter iter)
-	{
-		GLib.Value value = new GLib.Value ();
-		Model.GetValue (iter, IdColumn, ref value);
-		uint tag_id = (uint) value;
-		Tag tag = tag_store.Get (tag_id) as Tag;
-
-		SetBackground (renderer, tag);
-		(renderer as CellRendererToggle).Active = IsSelected (tag);
-	}
-
 	private void IconDataFunc (TreeViewColumn column, 
 				   CellRenderer renderer,
 				   TreeModel model,
@@ -297,7 +186,7 @@ public class TagSelectionWidget : TreeView {
 
 		// FIXME I can't set the Pixbuf to null, not sure if it's a GTK# bug...
 		if (tag.Icon != null)
-			(renderer as CellRendererPixbuf).Pixbuf = tag.Icon;
+			(renderer as CellRendererPixbuf).Pixbuf = tag.SizedIcon;
 		else
 			(renderer as CellRendererPixbuf).Pixbuf = empty_pixbuf;
 	}
@@ -560,9 +449,15 @@ public class TagSelectionWidget : TreeView {
 			}
 		}
 
-		if (expanded_tags.Count > 0)
-			FSpot.Preferences.Set (	FSpot.Preferences.EXPANDED_TAGS,
+#if GCONF_SHARP_2_18
+		FSpot.Preferences.Set (	FSpot.Preferences.EXPANDED_TAGS, (int []) expanded_tags.ToArray (typeof (int)));
+#else
+		if (expanded_tags.Count == 0)
+			expanded_tags.Add (-1);
+
+		FSpot.Preferences.Set (	FSpot.Preferences.EXPANDED_TAGS,
 						(int []) expanded_tags.ToArray (typeof (int)));
+#endif
 	}
 
 	public void EditSelectedTagName ()
@@ -571,7 +466,11 @@ public class TagSelectionWidget : TreeView {
 		if (rows.Length != 1)
 			return;
 		
-		SetCursor (rows[0], name_column, true);
+		//SetCursor (rows[0], NameColumn, true);
+		text_render.Editable = true;
+		text_render.Edited += HandleTagNameEdited;
+		SetCursor (rows[0], complete_column, true);
+		text_render.Editable = false;
 	}
 
 	// Override the default ButtonPressEvent because it is stupid and
@@ -608,8 +507,8 @@ public class TagSelectionWidget : TreeView {
 			HigMessageDialog md = new HigMessageDialog (MainWindow.Toplevel.Window,
 				DialogFlags.DestroyWithParent, 
 				MessageType.Warning, ButtonsType.Ok, 
-				Mono.Posix.Catalog.GetString ("Error renaming tag"),
-				Mono.Posix.Catalog.GetString ("This name is already in use"));
+				Catalog.GetString ("Error renaming tag"),
+				Catalog.GetString ("This name is already in use"));
 
 			md.Run ();
 			md.Destroy ();
@@ -620,13 +519,18 @@ public class TagSelectionWidget : TreeView {
 		tag.Name = args.NewText;
 		tag_store.Commit (tag);
 
+		text_render.Edited -= HandleTagNameEdited;
+
 		args.RetVal = true;
 		return;
 	}
 
-	TreeViewColumn check_column;
-	TreeViewColumn icon_column;
-	TreeViewColumn name_column;
+	//TreeViewColumn check_column;
+	//TreeViewColumn icon_column;
+	//TreeViewColumn name_column;
+	CellRendererPixbuf pix_render;
+	TreeViewColumn complete_column;
+	CellRendererText text_render;
 
 	// Constructor.
 	public TagSelectionWidget (TagStore tag_store)
@@ -635,27 +539,27 @@ public class TagSelectionWidget : TreeView {
 		HeadersVisible = false;
 		Selection.Mode = SelectionMode.Multiple;
 
-		CellRendererToggle toggle_renderer = new CellRendererToggle ();
-		toggle_renderer.Toggled += new ToggledHandler (OnCellToggled);
+		complete_column = new TreeViewColumn ();
+				
+		pix_render = new CellRendererPixbuf ();
+		complete_column.PackStart (pix_render, false);
+		complete_column.SetCellDataFunc (pix_render, new TreeCellDataFunc (IconDataFunc));
+		//complete_column.AddAttribute (pix_render, "pixbuf", OpenIconColumn);
 
-		check_column = AppendColumn ("check", toggle_renderer, new TreeCellDataFunc (CheckBoxDataFunc));
-		check_column.SortColumnId = 0;
+		//icon_column = AppendColumn ("icon", 
+		//, new TreeCellDataFunc (IconDataFunc));
+		//icon_column = AppendColumn ("icon", new CellRendererPixbuf (), new TreeCellDataFunc (IconDataFunc));
 
-		icon_column = AppendColumn ("icon", new CellRendererPixbuf (), new TreeCellDataFunc (IconDataFunc));
+		text_render = new CellRendererText ();
+		complete_column.PackStart (text_render, true);
+		complete_column.SetCellDataFunc (text_render, new TreeCellDataFunc (NameDataFunc));
 
-		CellRendererText tr = new CellRendererText ();
-		tr.Editable = true;
-		tr.Edited += HandleTagNameEdited;
-		tr.Mode = CellRendererMode.Editable;
+		AppendColumn (complete_column);
 
-		name_column = AppendColumn ("name", tr, new TreeCellDataFunc (NameDataFunc));
-		name_column.SortColumnId = NameColumn;
-		
 		this.tag_store = tag_store;
-		selection = new Hashtable ();
 
 		Update ();
-		
+
 		ExpandDefaults ();
 
 		tag_store.ItemsAdded += HandleTagsAdded;
@@ -664,7 +568,7 @@ public class TagSelectionWidget : TreeView {
 
 		// TODO make the search find tags that are not currently expanded
 		EnableSearch = true;
-		SearchColumn = 1;
+		SearchColumn = NameColumn;
 
 		// Transparent white
 		empty_pixbuf.Fill(0xffffff00);
