@@ -1,12 +1,15 @@
 using Gdk;
 using Gnome;
 using Gtk;
-using Mono.Posix;
+using Mono.Unix;
 using Mono.Data.SqliteClient;
 using System.Collections;
 using System.IO;
 using System;
-
+using Banshee.Database;
+using FSpot;
+using FSpot.Jobs;
+using FSpot.Query;
 
 // FIXME: This is to workaround the currently busted GTK# bindings.
 using System.Runtime.InteropServices;
@@ -27,9 +30,6 @@ public class PixbufSerializer {
 	[DllImport("libgdk_pixbuf-2.0-0.dll")]
 	static extern IntPtr gdk_pixdata_serialize(ref Gdk.Pixdata raw, out uint stream_length_p);
 
-	[DllImport("libglib-2.0-0.dll")]
-	static extern void g_free(IntPtr ptr);
-
 	public static byte [] Serialize (Pixbuf pixbuf)
 	{
 		Pixdata pixdata = new Pixdata ();
@@ -41,7 +41,7 @@ public class PixbufSerializer {
 		byte [] data = new byte [data_length];
 		Marshal.Copy (raw_data, data, 0, (int) data_length);
 		
-		g_free (raw_data);
+		GLib.Marshaller.Free (raw_data);
 
 		return data;
 	}
@@ -84,35 +84,75 @@ public class Tag : DbItem, IComparable {
 		}
 	}
 
-
-	// Icon.  If stock_icon_name is not null, then we save the name of the icon instead
+	// Icon.  If theme_icon_name is not null, then we save the name of the icon instead
 	// of the actual icon data.
 
-	private string stock_icon_name;
-	public string StockIconName {
+	private string theme_icon_name;
+	public string ThemeIconName {
 		set {
-			stock_icon_name = value;
-			Icon = PixbufUtils.LoadFromAssembly (value);
-		}
-
-		get {
-			return stock_icon_name;
+			theme_icon_name = value;
+			cached_icon_size = IconSize.Hidden;
+			icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, theme_icon_name, 48, (Gtk.IconLookupFlags)0);
 		}
+		get { return theme_icon_name; }
 	}
 
 	private Pixbuf icon;
 	public Pixbuf Icon {
 		set {
-			// If a custom icon is set, of course it means we are not using a stock
-			// icon.
-			stock_icon_name = null;
+			theme_icon_name = null;
+			if (icon != null)
+				icon.Dispose ();
 			icon = value;
+			cached_icon_size = IconSize.Hidden;
 		}
-		get {
-			return icon;
-		}
+		get { return icon; }
 	}
 
+	public enum IconSize {
+		Hidden = 0,
+		Small = 16,
+		Medium = 24,
+		Large = 48
+	};
+
+	private static IconSize tag_icon_size = IconSize.Large;
+	public static IconSize TagIconSize {
+		get { return tag_icon_size; }
+		set { tag_icon_size = value; }
+	}
+
+	private Pixbuf cached_icon;
+	private IconSize cached_icon_size = IconSize.Hidden;
+
+	// We can use a SizedIcon everywhere we were using an Icon
+	public Pixbuf SizedIcon {
+		get {
+			if (tag_icon_size == IconSize.Hidden) //Hidden
+				return null;
+			if (tag_icon_size == cached_icon_size)
+				return cached_icon;
+			if (theme_icon_name != null) { //Theme icon
+				if (cached_icon != null)
+					cached_icon.Dispose ();
+				cached_icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, theme_icon_name, (int) tag_icon_size, (Gtk.IconLookupFlags)0);
+
+				if (Math.Max (cached_icon.Width, cached_icon.Height) <= (int) tag_icon_size) 
+					return cached_icon;
+			}
+			if (Math.Max (icon.Width, icon.Height) >= (int) tag_icon_size) { //Don't upscale
+				if (cached_icon != null)
+					cached_icon.Dispose ();
+				cached_icon = icon.ScaleSimple ((int) tag_icon_size, (int) tag_icon_size, InterpType.Bilinear);
+				cached_icon_size = tag_icon_size;
+				return cached_icon;
+			}
+			else
+				return icon;
+		}	
+	}
+
+
 	// You are not supposed to invoke these constructors outside of the TagStore class.
 	public Tag (Category category, uint id, string name)
 		: base (id)
@@ -250,10 +290,10 @@ public class TagStore : DbStore {
 
 	private void SetIconFromString (Tag tag, string icon_string)
 	{
-		if (icon_string == null || icon_string == "")
+		if (icon_string == null || icon_string == String.Empty)
 			tag.Icon = null;
 		else if (icon_string.StartsWith (STOCK_ICON_DB_PREFIX))
-			tag.StockIconName = icon_string.Substring (STOCK_ICON_DB_PREFIX.Length);
+			tag.ThemeIconName = icon_string.Substring (STOCK_ICON_DB_PREFIX.Length);
 		else
 			tag.Icon = PixbufSerializer.Deserialize (Convert.FromBase64String (icon_string));
 	}
@@ -303,13 +343,10 @@ public class TagStore : DbStore {
 	// base class.
 	private void LoadAllTags ()
 	{
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
 
 		// Pass 1, get all the tags.
 
-		command.CommandText = "SELECT id, name, is_category, sort_priority, icon FROM tags";
-		SqliteDataReader reader = command.ExecuteReader ();
+		SqliteDataReader reader = Database.Query("SELECT id, name, is_category, sort_priority, icon FROM tags");
 
 		while (reader.Read ()) {
 			uint id = Convert.ToUInt32 (reader [0]);
@@ -330,12 +367,10 @@ public class TagStore : DbStore {
 		}
 
 		reader.Close ();
-		command.Dispose ();
 
 		// Pass 2, set the parents.
 
-		command.CommandText = "SELECT id, category_id FROM tags";
-		reader = command.ExecuteReader ();
+		reader = Database.Query("SELECT id, category_id FROM tags");
 
 		while (reader.Read ()) {
 			uint id = Convert.ToUInt32 (reader [0]);
@@ -354,7 +389,6 @@ public class TagStore : DbStore {
 
 		}
 		reader.Close ();
-		command.Dispose ();
 
 		if (FSpot.Core.Database.Meta.HiddenTagId.Value != null)
 			hidden = LookupInCache ((uint) FSpot.Core.Database.Meta.HiddenTagId.ValueAsInt) as Tag;
@@ -363,33 +397,28 @@ public class TagStore : DbStore {
 
 	private void CreateTable ()
 	{
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
 
-		command.CommandText =
-			"CREATE TABLE tags (                               " +
-			"	id            INTEGER PRIMARY KEY NOT NULL," +
-			"       name          TEXT UNIQUE,                 " +
-			"       category_id   INTEGER,			   " +
-			"       is_category   BOOLEAN,			   " +
-			"       sort_priority INTEGER,			   " +
-			"       icon          TEXT			   " +
-			")";
+		Database.ExecuteNonQuery ("CREATE TABLE tags (                            " +
+				   "	id            INTEGER PRIMARY KEY NOT NULL," +
+				   "       name          TEXT UNIQUE,                 " +
+				   "       category_id   INTEGER,			   " +
+				   "       is_category   BOOLEAN,			   " +
+				   "       sort_priority INTEGER,			   " +
+				   "       icon          TEXT			   " +
+				   ")");
 
-		command.ExecuteNonQuery ();
-		command.Dispose ();
 	}
 
 
 	private void CreateDefaultTags ()
 	{
 		Category favorites_category = CreateCategory (RootCategory, Catalog.GetString ("Favorites"));
-		favorites_category.StockIconName = "f-spot-favorite.png";
+		favorites_category.ThemeIconName = "emblem-favorite";
 		favorites_category.SortPriority = -10;
 		Commit (favorites_category);
 
 		Tag hidden_tag = CreateTag (RootCategory, Catalog.GetString ("Hidden"));
-		hidden_tag.StockIconName = "f-spot-hidden.png";
+		hidden_tag.ThemeIconName = "emblem-readonly";
 		hidden_tag.SortPriority = -9;
 		this.hidden = hidden_tag;
 		Commit (hidden_tag);
@@ -397,17 +426,17 @@ public class TagStore : DbStore {
 		FSpot.Core.Database.Meta.Commit (FSpot.Core.Database.Meta.HiddenTagId);
 
 		Tag people_category = CreateCategory (RootCategory, Catalog.GetString ("People"));
-		people_category.StockIconName = "f-spot-people.png";
+		people_category.ThemeIconName = "emblem-people";
 		people_category.SortPriority = -8;
 		Commit (people_category);
 
 		Tag places_category = CreateCategory (RootCategory, Catalog.GetString ("Places"));
-		places_category.StockIconName = "f-spot-places.png";
+		places_category.ThemeIconName = "emblem-places";
 		places_category.SortPriority = -8;
 		Commit (places_category);
 
 		Tag events_category = CreateCategory (RootCategory, Catalog.GetString ("Events"));
-		events_category.StockIconName = "f-spot-events.png";
+		events_category.ThemeIconName = "emblem-event";
 		events_category.SortPriority = -7;
 		Commit (events_category);
 	}
@@ -415,11 +444,11 @@ public class TagStore : DbStore {
 
 	// Constructor
 
-	public TagStore (SqliteConnection connection, bool is_new)
-		: base (connection, true)
+	public TagStore (QueuedSqliteDatabase database, bool is_new)
+		: base (database, true)
 	{
 		// The label for the root category is used in new and edit tag dialogs
-		root_category = new Category (null, 0, Mono.Posix.Catalog.GetString ("(None)"));
+		root_category = new Category (null, 0, Catalog.GetString ("(None)"));
 
 		if (! is_new) {
 			LoadAllTags ();
@@ -431,22 +460,17 @@ public class TagStore : DbStore {
 
 	private uint InsertTagIntoTable (Category parent_category, string name, bool is_category)
 	{
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
 
 		uint parent_category_id = parent_category.Id;
 
-		command.CommandText = String.Format
-			("INSERT INTO tags (name, category_id, is_category, sort_priority) " +
-			 "            VALUES ('{0}', {1}, {2}, 0)                          ",
-			 SqlString (name),
-			 parent_category_id,
-			 is_category ? 1 : 0);
+		int id = Database.Execute (new DbCommand ("INSERT INTO tags (name, category_id, is_category, sort_priority)"
+                          + "VALUES (:name, :category_id, :is_category, 0)",
+						  "name", name,
+						  "category_id", parent_category_id,
+						  "is_category", is_category ? 1 : 0));
 
-		command.ExecuteScalar ();
-		command.Dispose ();
 
-		return (uint) Connection.LastInsertRowId;
+		return (uint) id;
 	}
 
 	public Tag CreateTag (Category category, string name)
@@ -499,56 +523,54 @@ public class TagStore : DbStore {
 		
 		((Tag)item).Category = null;
 		
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
 
-		command.CommandText = String.Format ("DELETE FROM tags WHERE id = {0}", item.Id);
-		command.ExecuteNonQuery ();
+		Database.ExecuteNonQuery (new DbCommand ("DELETE FROM tags WHERE id = :id", "id", item.Id));
 
-		command.Dispose ();
 		EmitRemoved (item);
 	}
 
 
 	private string GetIconString (Tag tag)
 	{
-		if (tag.StockIconName != null)
-			return STOCK_ICON_DB_PREFIX + tag.StockIconName;
+		if (tag.ThemeIconName != null)
+			return STOCK_ICON_DB_PREFIX + tag.ThemeIconName;
 		if (tag.Icon == null)
-			return "";
+			return String.Empty;
 
 		byte [] data = PixbufSerializer.Serialize (tag.Icon);
 		return Convert.ToBase64String (data);
 	}
 
 	public override void Commit (DbItem item)
+	{
+		Commit (item, false);	
+	}
+
+	public void Commit (DbItem item, bool update_xmp)
 	{
 		Tag tag = item as Tag;
 
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-
-		command.CommandText = String.Format ("UPDATE tags SET          " +
-						     "    name = '{0}',        " +
-						     "    category_id = {1},   " +
-						     "    is_category = {2},   " +
-						     "    sort_priority = {3}, " +
-						     "    icon = '{4}'	       " +
-						     "WHERE id = {5}           ",
-						     SqlString (tag.Name),
-						     tag.Category.Id,
-						     tag is Category ? 1 : 0,
-						     tag.SortPriority,
-						     SqlString (GetIconString (tag)),
-						     tag.Id);
-		command.ExecuteNonQuery ();
-
-		command.Dispose ();
+		Database.ExecuteNonQuery (new DbCommand ("UPDATE tags SET name = :name, category_id = :category_id, "
+                    + "is_category = :is_category, sort_priority = :sort_priority, icon = :icon WHERE id = :id",
+						  "name", tag.Name,
+						  "category_id", tag.Category.Id,
+						  "is_category", tag is Category ? 1 : 0,
+						  "sort_priority", tag.SortPriority,
+						  "icon", GetIconString (tag),
+						  "id", tag.Id));
 		
 		EmitChanged (tag);
+
+		if (update_xmp && (bool)Preferences.Get(Preferences.METADATA_EMBED_IN_IMAGE)) {
+			Photo [] photos = Core.Database.Photos.Query (new Tag [] { tag });
+			foreach (Photo p in photos) {
+				SyncMetadataJob.Create (Core.Database.Jobs, p);
+			}
+		}
 	}
 
 
+
 #if TEST_TAG_STORE
 
 	private static void Dump (Category category, int indent)
