@@ -7,6 +7,8 @@ using System.Text;
 using System.Collections;
 using Mono.Unix;
 using Mono.GetOptions;
+using Mono.Addins;
+using Mono.Addins.Setup;
 
 namespace FSpot 
 {
@@ -82,7 +84,6 @@ public class Driver {
 
 		SetProcessName (Defines.PACKAGE);
 		
-		NDesk.DBus.BusG.Init();
 		try {
 			FSpotOptions options = new FSpotOptions ();
 			options.ProcessArgs (args);
@@ -115,6 +116,11 @@ public class Driver {
 				return;
 			}
 
+			try {
+				NDesk.DBus.BusG.Init();
+			} catch (Exception e) {
+				throw new ApplicationException ("F-Spot cannot find the Dbus session bus.  Make sure dbus is configured properly or start a new session for f-spot using \"dbus-launch f-spot\"", e);
+			}
 			/* 
 			 * FIXME we need to inialize gobject before making the dbus calls, we'll go 
 			 * ahead and do it like this for now.
@@ -123,6 +129,12 @@ public class Driver {
 					       Defines.VERSION, 
 					       Modules.UI, args);		
 			
+			Console.WriteLine ("Initializing Mono.Addins");
+			AddinManager.Initialize (FSpot.Global.BaseDirectory);
+			AddinManager.Registry.Update (null);
+			SetupService setupService = new SetupService (AddinManager.Registry);
+			setupService.Repositories.RegisterRepository (null, "http://addins.f-spot.org", false);
+
 			bool create = true;
 			while (control == null) {
 				try {
@@ -139,13 +151,17 @@ public class Driver {
 					if (control == null && create) {
 						create = false;
 						Gnome.Vfs.Vfs.Initialize ();
-						StockIcons.Initialize ();
 						
 						Catalog.Init ("f-spot", Defines.LOCALE_DIR);
-						Gtk.Window.DefaultIconList = new Gdk.Pixbuf [] {PixbufUtils.LoadFromAssembly ("f-spot-16.png"),
-												PixbufUtils.LoadFromAssembly ("f-spot-22.png"),
-												PixbufUtils.LoadFromAssembly ("f-spot-32.png"),
-												PixbufUtils.LoadFromAssembly ("f-spot-48.png")};
+						try {
+							Gtk.Window.DefaultIconList = new Gdk.Pixbuf [] {
+								GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "f-spot", 16, (Gtk.IconLookupFlags)0),
+								GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "f-spot", 22, (Gtk.IconLookupFlags)0),
+								GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "f-spot", 32, (Gtk.IconLookupFlags)0),
+								GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "f-spot", 48, (Gtk.IconLookupFlags)0)
+							};
+						} catch {}
+
 						core = new Core (options.view);
 						core.RegisterServer ();
 						
@@ -155,12 +171,14 @@ public class Driver {
 						control = core;
 					}
 				} catch (System.Exception e) {
-					System.Console.WriteLine ("XXXXX\n{0}\nXXXXX", e);
+					System.Console.WriteLine ("XXXXX{1}{0}{1}XXXXX", e, Environment.NewLine);
 					control = null;
 
 					if (core != null)
 						core.UnregisterServer ();
 				}
+				if (control == null)
+					System.Console.WriteLine ("Can't get a connection to the dbus. Trying again...");
 			}
 			
 			
